import 'package:flutter/foundation.dart';

abstract class Notifiable<T> {
  VoidCallback addListener(ValueChanged<T> listener);
}

class Notifier<T> extends Notifiable<T> {
  Notifier([T initialValue]) {
    _notifier = createValueNotifier(initialValue);
  }

  ValueNotifier<T> _notifier;

  @visibleForTesting
  ValueNotifier<T> createValueNotifier([T initialValue]) {
    return ValueNotifier<T>(initialValue);
  }

  VoidCallback addListener(ValueChanged<T> listener) {
    final _listener = () => listener(_notifier.value);
    _notifier.addListener(_listener);
    listener(_notifier.value);
    return () => _notifier.removeListener(_listener);
  }

  void push(T value) {
    _notifier.value = value;
  }

  void dispose() {
    _notifier.dispose();
  }
}
