part of 'framework.dart';

class _SuspenseVisibility extends MultiChildRenderObjectWidget {
  _SuspenseVisibility({Key key, this.controller, Widget child, Widget fallback})
      : super(key: key, children: [child, fallback]);

  final SuspenseController controller;

  @override
  RenderObject createRenderObject(BuildContext context) =>
      RenderCustom()..controller = controller;

  @override
  void updateRenderObject(BuildContext context, RenderCustom renderObject) {
    renderObject.controller = controller;
  }

  @override
  void didUnmountRenderObject(RenderCustom renderObject) {}
}

class RenderCustom extends RenderBox
    with ContainerRenderObjectMixin<RenderBox, _SuspenseVisibilityParentData> {
  VoidCallback _subscription;

  bool _loading;
  bool get loading => _loading;
  set loading(bool loading) {
    if (loading != _loading) {
      _loading = loading;
      markNeedsPaint();
    }
  }

  void dispose() {
    if (_subscription != null) {
      _subscription();
    }
  }

  SuspenseController _controller;
  SuspenseController get controller => _controller;

  set controller(SuspenseController controller) {
    assert(controller != null);
    if (controller != _controller) {
      if (_subscription != null) {
        _subscription();
      }
      _controller = controller;
      _subscription =
          controller.loading.addListener((value) => loading = value);
      markNeedsPaint();
    }
  }

  RenderBox get child => firstChild;
  RenderBox get fallback => lastChild;

  @override
  bool get sizedByParent => true;

  @override
  void performResize() {
    size = constraints.biggest;
  }

  @override
  void performLayout() {
    parentData = _SuspenseVisibilityParentData();
    child.layout(BoxConstraints.tight(constraints.biggest));
    fallback.layout(BoxConstraints.tight(constraints.biggest));
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    if (_loading) {
      context.paintChild(fallback, offset);
    } else {
      context.paintChild(child, offset);
    }
  }

  @override
  bool hitTestChildren(HitTestResult result, {Offset position}) {
    if (_loading) {
      return fallback.hitTest(result, position: position);
    } else {
      return child.hitTest(result, position: position);
    }
  }

  @override
  void visitChildrenForSemantics(visitor) {
    if (_loading) {
      visitor(child);
    } else {
      visitor(fallback);
    }
  }
}

class _SuspenseVisibilityParentData extends MultiChildLayoutParentData
    with ContainerParentDataMixin<RenderBox> {}

class _SuspenseVisibilityWrapper extends SingleChildRenderObjectWidget {
  _SuspenseVisibilityWrapper({Widget child}) : super(child: child);

  @override
  RenderObject createRenderObject(BuildContext context) =>
      _RenderSuspenseVisibilityWrapper();
}

class _RenderSuspenseVisibilityWrapper extends RenderProxyBox {
  final parentData = _SuspenseVisibilityParentData();
}
