import 'package:flutter/foundation.dart';
import 'package:flutter_suspense/src/notifier.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';

class ValueNotifierMock<T> extends Mock implements ValueNotifier<T> {}

class _Notifier<T> extends Notifier<T> {
  _Notifier(this.mock);

  ValueNotifier<T> mock;

  @override
  ValueNotifier<T> createValueNotifier([T intialValue]) => mock;
}

class MockListener<T> extends Mock {
  void call(T value);
}

void main() {
  group('notifier', () {
    test('addListener delegates to internal valueNotifier', () {
      final mock = ValueNotifierMock<int>();
      final notifier = _Notifier(mock);

      verifyZeroInteractions(mock);

      notifier.addListener((_) {});

      verify(mock.addListener(any))..called(1);
      verify(mock.value);
      verifyNoMoreInteractions(mock);
    });
    test('addListener result allow removing the listener', () {
      final mock = ValueNotifierMock<int>();
      final notifier = _Notifier(mock);

      verifyZeroInteractions(mock);

      final dispose = notifier.addListener((_) {});
      final capture = verify(mock.addListener(captureAny))..called(1);
      clearInteractions(mock);

      dispose();
      verify(mock.removeListener(capture.captured.first));
      verifyNoMoreInteractions(mock);
    });

    test('addListener calls listener immediatly', () {
      final notifier = Notifier(0);

      final listener = MockListener<int>();
      notifier.addListener(listener);
      verify(listener(0)).called(1);
      verifyNoMoreInteractions(listener);
    });

    test('pushing values call listener', () {
      final mock = ValueNotifierMock<int>();
      final notifier = _Notifier(mock);

      when(mock.value).thenReturn(0);

      notifier.addListener((_) {});
      clearInteractions(mock);

      notifier.push(42);
      verify(mock.value = 42);
    });
  });
}
